(* $Id: main.ml,v 1.2 2013-02-26 08:15:06 deraugla Exp $ *)
(* Command "diff" testing module Diff *)

open Printf;

value print_result a b chng1 chng2 = do {
  loop 0 0 where rec loop i j =
    if i = Array.length a || j = Array.length b then
      if i < Array.length a then do {
        printf "%dd%d\n" (i + 1) j;
        printf "< %s\n" a.(i);
        loop (i + 1) j
      }
      else if j < Array.length b then do {
        printf "%da%d\n" i (j + 1);
        printf "> %s\n" b.(j);
        loop i (j + 1)
      }
      else ()
    else
      match (chng1.(i), chng2.(j)) with
      [ (True, True) -> do {
          let i2 =
            loop (i + 1) where rec loop i =
              if i = Array.length a || not chng1.(i) then i else loop (i + 1)
          in
          let j2 =
            loop (j + 1) where rec loop j =
              if j = Array.length b || not chng2.(j) then j else loop (j + 1)
          in
          printf "%d" (i + 1);
          if i2 > i + 1 then printf ",%d" i2 else ();
          printf "c";
          printf "%d" (j + 1);
          if j2 > j + 1 then printf ",%d" j2 else ();
          printf "\n";
          for i = i to i2 - 1 do { printf "< %s\n" a.(i) };
          printf "---\n";
          for j = j to j2 - 1 do { printf "> %s\n" b.(j) };
          loop i2 j2
        }
      | (True, False) -> do {
          let i2 =
            loop (i + 1) where rec loop i =
              if i = Array.length a || not chng1.(i) then i else loop (i + 1)
          in
          printf "%d" (i + 1);
          if i2 > i + 1 then printf ",%d" i2 else ();
          printf "d%d\n" j;
          for i = i to i2 - 1 do { printf "< %s\n" a.(i) };
          loop i2 j
        }
      | (False, True) -> do {
          let j2 =
            loop (j + 1) where rec loop j =
              if j = Array.length b || not chng2.(j) then j else loop (j + 1)
          in
          printf "%da%d" i (j + 1);
          if j2 > j + 1 then printf ",%d" j2 else ();
          printf "\n";
          for j = j to j2 - 1 do { printf "> %s\n" b.(j) };
          loop i j2
        }
      | (False, False) ->
          if a.(i) != b.(j) then failwith "internal error"
          else loop (i + 1) (j + 1) ];
  flush stdout
};

value diff_them a b =
  let (chng1, chng2) = Diff.f a b in
  print_result a b chng1 chng2
;

value read_file f =
  let ic = if f = "-" then stdin else open_in f in
  loop [] 0 where rec loop list len =
    match try Some (input_line ic) with [ End_of_file -> None ] with
    [ Some line -> loop [line :: list] (len + 1)
    | None -> do {
        if f = "-" then () else close_in ic;
        let a = Array.create len "" in
        loop len list where rec loop i =
          fun
          [ [x :: l] -> do { a.(i-1) := x; loop (i - 1) l }
          | [] -> a ]
      } ]
;

value diff f1 f2 =
  let a1 = read_file f1 in
  let a2 = read_file f2 in
  diff_them a1 a2
;

value main () =
  let f1 = Sys.argv.(1) in
  let f2 = Sys.argv.(2) in
  diff f1 f2
;

main ();
